==> conc-2.0.1.tar.gz: conc-2.0.1/svr/log.c <==
#include <string.h>
#include "svr.h"
#include "log.h"
#include "svr-message.h"

typedef struct _Log {
    gchar *filename;
    GIOChannel *io;
    MplexChannel chan;
    Server *server;
} Log;

static void
log_tree_free(gpointer data) {
    Log *log = data;

    g_free(log->filename);
    g_io_channel_unref(log->io);
    /* FIXME: do something with the mplex channel? */
    g_free(log);
}

static void
log_open(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Log *log;
    Server *server = data;
    MessageArg *filename;
    MessageArg *a;
    GIOChannel *io;
    GError *e = NULL;
    MplexChannel chan;

    g_assert(server);

    g_assert(message_arg_list_explode(arg,
		"file", MESSAGE_ARG_STRING, &filename,
		NULL, 0, NULL) == TRUE);
    g_printerr("attempting to open log %s\n", filename->string->str);
    io = g_io_channel_new_file(filename->string->str, "a", &e);
    if (io == NULL) {
	int errcode = 1;

	g_printerr(_("failed to open log file %s\n"), filename->string->str);
	if (g_error_matches(e, G_FILE_ERROR, G_FILE_ERROR_EXIST) == TRUE) {
	    errcode = 0;
	}
	a = message_arg_error(errcode, "%s", e->message);
	g_clear_error(&e);
	if (message_respond(q, id, a, &e) == FALSE) {
	    g_printerr(_("failed to send reponse (%s)\n"), e->message);
	    g_clear_error(&e);
	}
	message_arg_free(a);
	return;
    }
    if (mplex_connect(server->mplex, 0, &chan, io, &e) == FALSE) {
	g_printerr(_("failed to connect log to mplex (%s)\n"), e->message);
	g_clear_error(&e);
	return;
    }
    log = g_new0(Log, 1);
    log->filename = g_strdup(filename->string->str);
    log->io = io;
    log->chan = chan;
    log->server = server;
    g_tree_insert(server->logs, g_strdup(log->filename), log);
    a = message_arg_uint16("channel", chan);
    if (message_respond(q, id, a, &e) == FALSE) {
	g_printerr(_("failed to send reponse (%s)\n"), e->message);
	g_clear_error(&e);
    }
    message_arg_free(a);
}

extern void
log_register_message_handlers(Server *server) {
    GError *e = NULL;

    g_assert(server);

    if (server->msg_q == NULL) {
	return;
    }
    if (message_handler_register(server->msg_q,
		SERVER_MESSAGE_OPEN_LOG, log_open,
		server, &e) == FALSE) {
	g_printerr(_("could not register handler %s (%s)\n"),
		SERVER_MESSAGE_OPEN_LOG, e->message);
    }
}

extern GTree *
log_tree_init(void) {
    return g_tree_new_full((GCompareDataFunc)strcmp, NULL,
	    g_free, log_tree_free);
}

==> conc-2.0.3.tar.gz: conc-2.0.3/svr/log.c <==
#include <string.h>
#include "svr.h"
#include "log.h"
#include "svr-message.h"

typedef struct _Log {
    gchar *filename;
    GIOChannel *io;
    MplexChannel chan;
    Server *server;
} Log;

static void
log_hash_free(gpointer data) {
    Log *log = data;
    GError *e = NULL;

    if (mplex_close(log->server->mplex, log->chan, &e) == FALSE) {
	g_printerr(_("failed to close mplex channel (%s)\n"), e->message);
	g_clear_error(&e);
    }
    g_free(log->filename);
    g_free(log);
}

static void
log_closed(Mplex mplex,
	MplexChannel chan,
	gpointer data) {
    Log *log = data;

    g_printerr(_("log %s closed\n"), log->filename);
    g_hash_table_remove(log->server->logs, log->filename);
}

static void
log_open(MessageQueue q,
	const gchar *name,
	MessageID id,
	const MessageArg *arg,
	gpointer data) {
    Log *log;
    Server *server = data;
    MessageArg *filename;
    MessageArg *a;
    GIOChannel *io;
    GError *e = NULL;
    MplexChannel chan;

    g_assert(server);

    g_assert(message_arg_list_explode(arg,
		"file", MESSAGE_ARG_STRING, &filename,
		NULL, 0, NULL) == TRUE);
    g_printerr("attempting to open log %s\n", filename->string->str);
    io = g_io_channel_new_file(filename->string->str, "a", &e);
    if (io == NULL) {
	int errcode = 1;

	g_printerr(_("failed to open log file %s\n"), filename->string->str);
	if (g_error_matches(e, G_FILE_ERROR, G_FILE_ERROR_EXIST) == TRUE) {
	    errcode = 0;
	}
	a = message_arg_error(errcode, "%s", e->message);
	g_clear_error(&e);
	if (message_respond(q, id, a, &e) == FALSE) {
	    g_printerr(_("failed to send reponse (%s)\n"), e->message);
	    g_clear_error(&e);
	}
	message_arg_free(a);
	return;
    }
    log = g_new0(Log, 1);
    if (mplex_connect(server->mplex, 0, &chan, io, log_closed, log, &e)
	    == FALSE) {
	g_printerr(_("failed to connect log to mplex (%s)\n"), e->message);
	g_clear_error(&e);
	return;
    }
    log->filename = g_strdup(filename->string->str);
    log->io = io;
    log->chan = chan;
    log->server = server;
    g_hash_table_insert(server->logs, g_strdup(log->filename), log);
    a = message_arg_uint16("channel", chan);
    if (message_respond(q, id, a, &e) == FALSE) {
	g_printerr(_("failed to send reponse (%s)\n"), e->message);
	g_clear_error(&e);
    }
    message_arg_free(a);
}

extern void
log_register_message_handlers(Server *server) {
    GError *e = NULL;

    g_assert(server);

    if (server->logs == NULL) {
	server->logs = g_hash_table_new_full(g_str_hash, g_str_equal,
		g_free, log_hash_free);
    }
    if (server->msg_q == NULL) {
	return;
    }
    if (message_handler_register(server->msg_q,
		SERVER_MESSAGE_OPEN_LOG, log_open,
		server, &e) == FALSE) {
	g_printerr(_("could not register handler %s (%s)\n"),
		SERVER_MESSAGE_OPEN_LOG, e->message);
    }
}
